@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2010-2013  Tetsuo Handa <penguin-kernel@I-love.SAKURA.ne.jp>
  *
- * Version: 0.2.1   2017/02/20
+ * Version: 0.2.2   2017/09/17
  */
 
 #include "caitsith.h"
@@ -40,7 +40,7 @@ struct caitsith_exports caitsith_exports;
 /* Function pointers originally registered by register_security(). */
 static struct security_operations original_security_ops /* = *security_ops; */;
 
-#define wait_ready(op) while (!original_security_ops.op)
+#define wait_ready(op) while (!original_security_ops.op) smp_rmb()
 
 #if !defined(CONFIG_CAITSITH_DEBUG)
 #define cs_debug_trace(pos) do { } while (0)
