@@ -18,6 +18,29 @@
 
 #include "probe.h"
 
+static void *__init check_function_address(void *ptr, char *symbol)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
+	static char buf[KSYM_SYMBOL_LEN];
+	const int len = strlen(symbol);
+#endif
+
+	if (!ptr) {
+		printk(KERN_ERR "Can't resolve %s().\n", symbol);
+		return NULL;
+	}
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
+	snprintf(buf, sizeof(buf), "%pS", ptr);
+	if (strncmp(buf, symbol, len) || strncmp(buf + len, "+0x0/", 5)) {
+		printk(KERN_ERR "Guessed %s is %s\n", symbol, buf);
+		return NULL;
+	}
+#else
+	printk(KERN_INFO "%s=%p\n", symbol, ptr);
+#endif
+	return ptr;
+}
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24) || LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
 
 /**
@@ -53,6 +76,7 @@ static int __init probe_kernel_read(struct file *file, unsigned long offset,
 	return kernel_read(file, offset, addr, count);
 #else
 	loff_t pos = offset;
+
 	return kernel_read(file, addr, count, &pos);
 #endif
 }
@@ -202,6 +226,7 @@ static void * __init probe_security_hook_heads_on_arm64(unsigned int *base)
 	for (i = 0; i < 32; ip++, i++) {
 		unsigned long tmp;
 		unsigned long offset;
+
 		/*
 		 * Find
 		 *   adrp Xd, #imm21
@@ -250,6 +275,7 @@ static void * __init probe_security_hook_heads_on_arm(unsigned int *base)
 	const unsigned long addr = (unsigned long) &probe_dummy_security_hook_heads;
 	const unsigned long offset = (unsigned long) &probe_dummy_security_hook_heads.bprm_committed_creds - addr;
 	unsigned int *ip = (unsigned int *) probe_security_bprm_committed_creds;
+
 	for (i = 0; i < 32; ip++, i++) {
 		if (*(ip + 2 + ((*ip & 0xFFF) >> 2)) != addr)
 			continue;
@@ -467,7 +493,14 @@ struct security_hook_heads * __init probe_security_hook_heads(void)
 	const unsigned int offset = offsetof(struct security_hook_heads,
 					     bprm_committed_creds);
 	void *cp;
+	struct security_hook_heads *shh;
+	struct security_hook_list *entry;
+	void *cap = probe_find_symbol(" cap_bprm_set_creds\n");
 
+	/* Get location of cap_bprm_set_creds(). */
+	cap = check_function_address(cap, "cap_bprm_set_creds");
+	if (!cap)
+		return NULL;
 	/* Guess "struct security_hook_heads security_hook_heads;". */
 	cp = probe_find_variable(probe_security_bprm_committed_creds,
 				 ((unsigned long)
@@ -479,9 +512,13 @@ struct security_hook_heads * __init probe_security_hook_heads(void)
 		return NULL;
 	}
 	/* This should be "struct security_hook_heads security_hook_heads;". */
-	cp = ((void *) (*(unsigned long *) cp)) - offset;
-	printk(KERN_INFO "security_hook_heads=%p\n", cp);
-	return cp;
+	shh = ((void *) (*(unsigned long *) cp)) - offset;
+	list_for_each_entry(entry, &shh->bprm_set_creds, list)
+		if (entry->hook.bprm_set_creds == cap)
+			return shh;
+	printk(KERN_ERR "Guessed security_hook_heads is 0x%lx\n",
+	       (unsigned long) shh);
+	return NULL;
 }
 
 #else
@@ -543,12 +580,7 @@ void * __init probe_find_task_by_vpid(void)
 #else
 	ptr = __symbol_get("find_task_by_vpid");
 #endif
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve find_task_by_vpid().\n");
-		return NULL;
-	}
-	printk(KERN_INFO "find_task_by_vpid=%p\n", ptr);
-	return ptr;
+	return check_function_address(ptr, "find_task_by_vpid");
 }
 
 /**
@@ -565,12 +597,7 @@ void * __init probe_find_task_by_pid_ns(void)
 #else
 	ptr = __symbol_get("find_task_by_pid_ns");
 #endif
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve find_task_by_pid_ns().\n");
-		return NULL;
-	}
-	printk(KERN_INFO "find_task_by_pid_ns=%p\n", ptr);
-	return ptr;
+	return check_function_address(ptr, "find_task_by_pid_ns");
 }
 
 #endif
@@ -805,12 +832,7 @@ void * __init probe___d_path(void)
 {
 	void *ptr = probe_find_symbol(" __d_path\n");
 
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve __d_path().\n");
-		return NULL;
-	}
-	printk(KERN_INFO "__d_path=%p\n", ptr);
-	return ptr;
+	return check_function_address(ptr, "__d_path");
 }
 
 #endif
@@ -826,12 +848,7 @@ void * __init probe_d_absolute_path(void)
 {
 	void *ptr = probe_find_symbol(" d_absolute_path\n");
 
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve d_absolute_path().\n");
-		return NULL;
-	}
-	printk(KERN_INFO "d_absolute_path=%p\n", ptr);
-	return ptr;
+	return check_function_address(ptr, "d_absolute_path");
 }
 
 #endif
@@ -847,12 +864,7 @@ void * __init probe_ksize(void)
 {
 	void *ptr = probe_find_symbol(" ksize\n");
 
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve ksize().\n");
-		return NULL;
-	}
-	printk(KERN_INFO "ksize=%p\n", ptr);
-	return ptr;
+	return check_function_address(ptr, "ksize");
 }
 
 #endif
